@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2023 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2025 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -41,7 +41,8 @@ NOEXPORT void log_queue(SERVICE_OPTIONS *, int, char *, char *, char *);
 NOEXPORT void log_raw(SERVICE_OPTIONS *, int, char *, char *, char *);
 NOEXPORT void safestring(char *);
 
-static DISK_FILE *outfile=NULL;
+DISK_FILE *outfile=NULL;
+
 static struct LIST { /* single-linked list of log lines */
     struct LIST *next;
     SERVICE_OPTIONS *opt;
@@ -197,11 +198,7 @@ void s_vlog(int level, const char *format, va_list ap) {
         len=vsnprintf(NULL, 0, format, ap);
         if(len>1024)
             len=1024;
-#ifdef USE_WIN32
-        text=_alloca((size_t)len+1);
-#else
         text=alloca((size_t)len+1);
-#endif
         len=vsnprintf(text, (size_t)len+1, format, aq);
         va_end(aq);
         while(len>0 && text[len-1]=='\n')
@@ -287,29 +284,25 @@ NOEXPORT void log_raw(SERVICE_OPTIONS *opt,
     switch(log_mode) {
     case LOG_MODE_CONFIGURED:
         size=strlen(stamp)+strlen(id)+strlen(text)+4;
-#ifdef USE_WIN32
-        line=_alloca(size);
-#else
         line=alloca(size);
-#endif
         snprintf(line, size, "%s %s: %s", stamp, id, text);
         if(level<=opt->log_level) {
 #if !defined(USE_WIN32) && !defined(__vms)
             if(global_options.option.log_syslog)
                 syslog(level, "%s: %s", id, text);
 #endif /* USE_WIN32, __vms */
-            if(outfile)
+            if(outfile) {
                 file_putline_newline(outfile, line);
+#ifndef USE_OS_THREADS
+                file_flush(outfile);
+#endif /* !USE_OS_THREADS */
+            }
         }
         break;
     case LOG_MODE_ERROR:
         /* don't log the id or the time stamp */
         size=strlen(text)+5;
-#ifdef USE_WIN32
-        line=_alloca(size);
-#else
         line=alloca(size);
-#endif
         if(level>=0 && level<=7) /* just in case */
             snprintf(line, size, "[%c] %s", "***!:.  "[level], text);
         else
@@ -410,8 +403,10 @@ void fatal_debug(const char *txt, const char *file, int line) {
     snprintf(msg, sizeof msg, /* with newline */
         "INTERNAL ERROR: %s at %s, line %d\n", txt, file, line);
 
-    if(outfile)
+    if(outfile) {
         file_putline_nonewline(outfile, msg);
+        file_flush(outfile);
+    }
 
 #ifndef USE_WIN32
     if(log_mode!=LOG_MODE_CONFIGURED || global_options.option.log_stderr) {
