@@ -3,7 +3,7 @@
 import logging
 import os
 import pathlib
-from plugin_collection import Plugin
+from plugin_collection import Plugin, ERR_CONN_RESET
 from maketest import (
     Config,
     StunnelAcceptConnect
@@ -35,7 +35,7 @@ class CertTest(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             "Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -77,7 +77,7 @@ class FailureCertTest(StunnelTest):
         self.events.count = 1
         self.events.success = [
             "Client received unexpected message",
-            "\[Errno 104\] Connection reset by peer"
+            ERR_CONN_RESET
         ]
         self.events.failure = [
             "peer did not return a certificate",
@@ -86,7 +86,7 @@ class FailureCertTest(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             "Redirecting connection",
-            #"\[Errno 104\] Connection reset by peer",
+            #ERR_CONN_RESET,
             #"Connection lost",
             #"Client received unexpected message",
             "Server received unexpected message",
