@@ -2,7 +2,7 @@
 
 import logging
 import pathlib
-from plugin_collection import Plugin
+from plugin_collection import Plugin, ERR_CONN_RESET
 from maketest import (
     Config,
     StunnelAcceptConnect
@@ -37,7 +37,7 @@ class RedirectWrongPeerCert(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             #"Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -88,7 +88,7 @@ class RedirectNoPeerCert(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             #"Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -136,7 +136,7 @@ class NoRedirectCorrectPeerCert(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             "Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -186,7 +186,7 @@ class RedirectWrongChainCert(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             #"Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -237,7 +237,7 @@ class RedirectNoChainCert(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             #"Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -285,7 +285,7 @@ class NoRedirectCorrectChainCert(StunnelTest):
             "unsupported protocol",
             "TLS accepted: previous session reused",
             "Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
