@@ -3,7 +3,7 @@
 import logging
 import os
 import pathlib
-from plugin_collection import Plugin
+from plugin_collection import Plugin, ERR_CONN_RESET
 from maketest import (
     Config,
     ClientConnectExec
@@ -50,7 +50,7 @@ class RetryDelay(StunnelTest):
             "unsupported protocol",
             #"TLS accepted: previous session reused",
             "Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -70,7 +70,7 @@ class RetryDelay(StunnelTest):
     syslog = no
 
     delay = yes
-    retry = yes
+    retry = 10
 
     [{service}]
     client = yes
@@ -130,7 +130,7 @@ class RetryNoDelay(StunnelTest):
             "unsupported protocol",
             #"TLS accepted: previous session reused",
             "Redirecting connection",
-            "\[Errno 104\] Connection reset by peer",
+            ERR_CONN_RESET,
             "Connection lost",
             "Client received unexpected message",
             "Server received unexpected message",
@@ -151,7 +151,7 @@ class RetryNoDelay(StunnelTest):
 
     delay = no
     failover = rr
-    retry = yes
+    retry = 10
 
     [{service}]
     client = yes
